/* This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
/** library for receiving NMEA-0183 like messages over a Bluetooth module (API)
 *  @file radio_bluetooth.h
 *  @author King Kévin <kingkevin@cuvoodoo.info>
 *  @date 2017
 *  @note peripherals used: USART @ref radio_bluetooth_usart
 */
#pragma once

/** has the message been transmitted */
extern volatile bool radio_bluetooth_transmitted;
/** has a message been received */
extern volatile bool radio_bluetooth_received;
/** last valid message received ((including Carriage Return and Line Feed ending and NULL termination) */
extern volatile char radio_bluetooth_message[82+1];

/** setup USART peripheral for Bluetooth communication */
void radio_bluetooth_setup(void);
/** transmit message
 *  @param[in] message message to transmit (must be NULL terminated)
 *  @return is message will be transmitted (e.g. is not too long)
 *  @note blocking is a transmission is already ongoing
 */
bool radio_bluetooth_transmit(const char* message);
