/* This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
/** library for receiving forumslader NMEA-0183 like messages (API)
 *  @file sensor_forumslader.h
 *  @author King Kévin <kingkevin@cuvoodoo.info>
 *  @date 2017
 *  @note peripherals used: USART @ref sensor_forumslader_usart
 */
#pragma once

/** has the forumslader message been transmitted */
extern volatile bool sensor_forumslader_transmitted;
/** has a forumslader message been received */
extern volatile bool sensor_forumslader_received;
/** last valid forumslader message received (including Carriage Return and Line Feed ending and NULL termination) */
extern volatile char sensor_forumslader_message[82+1];

/** setup USART peripheral for forumslader communication */
void sensor_forumslader_setup(void);
/** transmit forumslader message
 *  @param[in] message message to transmit (must be NULL terminated)
 *  @return is message will be transmitted (e.g. is not too long)
 *  @note blocking is a transmission is already ongoing
 */
bool sensor_forumslader_transmit(const char* message);
